@@ -35,8 +35,8 @@ from nuitka.importing import Importing, Recursion
 from nuitka.Options import getPythonFlags
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import (
+    getPythonABI,
     getSupportedPythonVersions,
-    isStaticallyLinkedPython,
     isUninstalledPython,
     python_version,
     python_version_str
@@ -54,7 +54,7 @@ from nuitka.utils.FileOperations import (
 
 from . import ModuleRegistry, Options, TreeXML
 from .build import SconsInterface
-from .codegen import CodeGeneration, ConstantCodes
+from .codegen import CodeGeneration, ConstantCodes, Reports
 from .finalizations import Finalization
 from .freezer.BytecodeModuleFreezer import generateBytecodeFrozenCode
 from .freezer.Standalone import copyUsedDLLs, detectEarlyImports
@@ -167,14 +167,6 @@ def dumpTreeXML(tree):
     TreeXML.dump(xml_root)
 
 
-def displayTree(tree): # pragma: no cover
-    # Import only locally so the Qt4 dependency doesn't normally come into play
-    # when it's not strictly needed.
-    from .gui import TreeDisplay
-
-    TreeDisplay.displayTreeInspector(tree)
-
-
 def getTreeFilenameWithSuffix(tree, suffix):
     return tree.getOutputFilename() + suffix
 
@@ -216,25 +208,35 @@ def getResultBasepath(main_module):
 
 
 def getResultFullpath(main_module):
+    """ Get the final output binary result full path.
+
+    """
+
     result = getResultBasepath(main_module)
 
     if Options.shallMakeModule():
         result += Utils.getSharedLibrarySuffix()
     else:
-        result += ".exe"
+        if Options.getOutputFilename() is not None:
+            result = Options.getOutputFilename()
+        elif Utils.getOS() == "Windows":
+            result += ".exe"
+        elif not Options.isStandaloneMode():
+            result += ".bin"
 
     return result
 
 
 def cleanSourceDirectory(source_dir):
+    extensions = (
+        ".bin", ".c", ".cpp", ".exp", ".h",
+        ".lib", ".manifest", ".o", ".obj",
+        ".os", ".rc", ".res", ".S"
+    )
+
     if os.path.isdir(source_dir):
         for path, _filename in listDir(source_dir):
-            if hasFilenameExtension(
-                path       = path,
-                extensions = (".c", ".h", ".o", ".os", ".obj",
-                              ".bin", ".res", ".rc", ".S", ".cpp",
-                              ".manifest")
-            ):
+            if hasFilenameExtension(path, extensions):
                 deleteFile(path, must_exist = True)
     else:
         makePath(source_dir)
@@ -422,7 +424,7 @@ def makeSourceDirectory(main_module):
             if not os.path.isdir(target_dir):
                 makePath(target_dir)
 
-            shutil.copy(
+            shutil.copyfile(
                 module.getFilename(),
                 target_filename
             )
@@ -476,17 +478,8 @@ def _asBoolStr(value):
 
 def runScons(main_module, quiet):
     # Scons gets transported many details, that we express as variables, and
-    # have checks for them, leading to many branches, pylint: disable=too-many-branches,too-many-statements
-
-    if hasattr(sys, "abiflags"):
-        abiflags = sys.abiflags
-
-        if Options.isPythonDebug() or \
-           hasattr(sys, "getobjects"):
-            if not abiflags.startswith('d'):
-                abiflags = 'd' + abiflags
-    else:
-        abiflags = None
+    # have checks for them, leading to many branches and statements,
+    # pylint: disable=too-many-branches,too-many-statements
 
     options = {
         "name"            : os.path.basename(
@@ -513,6 +506,9 @@ def runScons(main_module, quiet):
         )
     }
 
+    if not Options.shallMakeModule():
+        options["result_exe"] = getResultFullpath(main_module)
+
     # Ask Scons to cache on Windows, except where the directory is thrown
     # away. On non-Windows you can should use ccache instead.
     if not Options.isRemoveBuildDir() and Utils.getOS() == "Windows":
@@ -532,9 +528,6 @@ def runScons(main_module, quiet):
        isUninstalledPython():
         options["uninstalled_python"] = "true"
 
-    if isStaticallyLinkedPython():
-        options["static_python"] = "true"
-
     if ModuleRegistry.getUncompiledTechnicalModules():
         options["frozen_modules"] = str(
             len(ModuleRegistry.getUncompiledTechnicalModules())
@@ -543,7 +536,7 @@ def runScons(main_module, quiet):
     if Options.isShowScons():
         options["show_scons"] = "true"
 
-    if Options.isMingw():
+    if Options.isMingw64():
         options["mingw_mode"] = "true"
 
     if Options.getMsvcVersion():
@@ -588,6 +581,7 @@ def runScons(main_module, quiet):
     if python_version >= 370 and sys.flags.utf8_mode:
         options["python_sysflag_utf8"] = "true"
 
+    abiflags = getPythonABI()
     if abiflags:
         options["abiflags"] = abiflags
 
@@ -718,6 +712,9 @@ def compileTree(main_module):
     if Options.isShowMemory():
         InstanceCounters.printStats()
 
+    if Options.isDebug():
+        Reports.doMissingOptimizationReport()
+
     if Options.shallNotDoExecCCompilerCall():
         return True, {}
 
@@ -736,8 +733,7 @@ def handleSyntaxError(e):
     # versions he wants, tell him about the potential version problem.
     error_message = SyntaxErrors.formatOutput(e)
 
-    if not Options.isFullCompat() and \
-       Options.getIntendedPythonVersion() is None:
+    if not Options.isFullCompat():
         if python_version < 300:
             suggested_python_version_str = getSupportedPythonVersions()[-1]
         else:
@@ -803,12 +799,9 @@ def main():
         handleSyntaxError(e)
 
     if Options.shallDumpBuiltTreeXML():
-        # XML only.
+        # XML output only.
         for module in ModuleRegistry.getDoneModules():
             dumpTreeXML(module)
-    elif Options.shallDisplayBuiltTree():
-        # GUI only.
-        displayTree(main_module)
     else:
         # Make the actual compilation.
         result, options = compileTree(
@@ -826,7 +819,7 @@ def main():
             sys.exit(0)
 
         if Options.isStandaloneMode():
-            binary_filename = options["result_name"] + ".exe"
+            binary_filename = options["result_exe"]
 
             standalone_entry_points.insert(
                 0,
@@ -877,7 +870,7 @@ def main():
             )
 
         # Modules should not be executable, but Scons creates them like it, fix
-        # it up here.
+        # it up here. TODO: Move inside scons file and avoid subprocess call.
         if Utils.getOS() != "Windows" and Options.shallMakeModule():
             subprocess.call(
                 (
