@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -38,8 +38,6 @@ from nuitka.utils.CStrings import encodePythonIdentifierToC
 from nuitka.utils.FileOperations import relpath
 
 from .Checkers import checkStatementsSequenceOrNone
-from .ConstantRefNodes import makeConstantRefNode
-from .ExpressionBases import ExpressionBase
 from .FutureSpecs import FutureSpec, fromFlags
 from .IndicatorMixins import EntryPointMixin, MarkNeedsAnnotationsMixin
 from .NodeBases import (
@@ -52,6 +50,8 @@ from .NodeBases import (
 
 
 class PythonModuleBase(NodeBase):
+    # Base classes can be abstract, pylint: disable=abstract-method
+
     __slots__ = "name", "package_name", "package"
 
     def __init__(self, name, package_name, source_ref):
@@ -104,12 +104,12 @@ class PythonModuleBase(NodeBase):
                 module_name    = self.package_name,
                 parent_package = None,
                 level          = 1,
-                warn           = python_version < 330
+                warn           = python_version < 300
             )
 
             # TODO: Temporary, if we can't find the package for Python3.3 that
             # is semi-OK, maybe.
-            if python_version >= 330 and not package_filename:
+            if python_version >= 300 and not package_filename:
                 return []
 
             if self.package_name == "uniconvertor.app.modules":
@@ -264,45 +264,62 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
     def getFutureSpec(self):
         return self.future_spec
 
-    def asGraph(self, computation_counter):
-        from graphviz import Digraph # @UnresolvedImport pylint: disable=I0021,import-error
+    def asGraph(self, graph, desc):
+        graph = graph.add_subgraph(
+            name    = "cluster_%s" % desc,
+            comment = "Graph for %s" % self.getName()
+        )
 
-        graph = Digraph("cluster_%d" % computation_counter, comment = "Graph for %s" % self.getName())
-        graph.body.append("style=filled")
-        graph.body.append("color=lightgrey")
-        graph.body.append("label=Iteration_%d" % computation_counter)
+#        graph.body.append("style=filled")
+#        graph.body.append("color=lightgrey")
+#        graph.body.append("label=Iteration_%d" % desc)
 
 
-        def makeTraceNodeName(variable_trace):
-            return "%d/ %s %s %s" % (
-                computation_counter,
-                variable_trace.getVariable(),
-                variable_trace.getVersion(),
+        def makeTraceNodeName(variable, version, variable_trace):
+            return "%s/ %s %s %s" % (
+                desc,
+                variable.getName(),
+                version,
                 variable_trace.__class__.__name__
             )
 
         for function_body in self.active_functions:
             trace_collection = function_body.trace_collection
 
-            for (_variable, _version), variable_trace in trace_collection.getVariableTracesAll().items():
-                node = makeTraceNodeName(variable_trace)
+            node_names = {}
+
+            for (variable, version), variable_trace in trace_collection.getVariableTracesAll().items():
+                node_name = makeTraceNodeName(variable, version, variable_trace)
+
+                node_names[variable_trace] = node_name
+
+            for (variable, version), variable_trace in trace_collection.getVariableTracesAll().items():
+                node_name = node_names[variable_trace]
 
                 previous = variable_trace.getPrevious()
 
-                if variable_trace.hasDefiniteUsages():
-                    graph.attr("node", style = "filled", color = "blue")
-                elif variable_trace.hasPotentialUsages():
-                    graph.attr("node", style = "filled", color = "yellow")
-                else:
-                    graph.attr("node", style = "filled", color = "red")
+                attrs = {
+                    "style" : "filled",
+                }
 
-                graph.node(node)
+                if variable_trace.hasDefiniteUsages():
+                    attrs["color"] = "blue"
+                elif variable_trace.hasPotentialUsages():
+                    attrs["color"] = "yellow"
+                else:
+                    attrs["color"] = "red"
+
+                graph.add_node(node_name, **attrs)
 
                 if type(previous) is tuple:
-                    for previous in previous:
-                        graph.edge(makeTraceNodeName(previous), node)
+                    for prev_trace in previous:
+                        graph.add_edge(node_names[prev_trace], node_name)
+
+                        assert prev_trace is not variable_trace
+
                 elif previous is not None:
-                    graph.edge(makeTraceNodeName(previous), node)
+                    assert previous is not variable_trace
+                    graph.add_edge(node_names[previous], node_name)
 
         return graph
 
@@ -392,9 +409,12 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
                function_body.isExpressionCoroutineObjectBody() or \
                function_body.isExpressionAsyncgenObjectBody()
 
-        if function_body not in self.active_functions:
+        result = function_body not in self.active_functions
+        if result:
             self.active_functions.add(function_body)
 
+        return result
+
     def getUsedFunctions(self):
         return self.active_functions
 
@@ -461,20 +481,16 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
         for function in self.getUsedFunctions():
             yield function.trace_collection
 
-    def markAsLocalsDict(self):
-        # Does not apply to modules.
-        pass
-
-    def hasLocalsDict(self):
-        return False
-
     def isUnoptimized(self):
+        # Modules don't do this, pylint: disable=no-self-use
         return False
 
     def getLocalVariables(self):
+        # Modules don't do this, pylint: disable=no-self-use
         return ()
 
     def getUserLocalVariables(self):
+        # Modules don't do this, pylint: disable=no-self-use
         return ()
 
     def getOutlineLocalVariables(self):
@@ -491,6 +507,7 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
         return tuple(result)
 
     def hasClosureVariable(self, variable):
+        # Modules don't do this, pylint: disable=no-self-use,unused-argument
         return False
 
     def removeUserVariable(self, variable):
@@ -503,6 +520,11 @@ class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
                 outline.removeUserVariable(variable)
                 break
 
+    @staticmethod
+    def getFunctionLocalsScope():
+        """ Modules have no locals scope. """
+        return None
+
 
 class CompiledPythonPackage(CompiledPythonModule):
     kind = "COMPILED_PYTHON_PACKAGE"
@@ -520,7 +542,12 @@ class CompiledPythonPackage(CompiledPythonModule):
         )
 
     def getOutputFilename(self):
-        return os.path.dirname(self.getFilename())
+        result = self.getFilename()
+
+        if os.path.isdir(result):
+            return result
+        else:
+            return os.path.dirname(self.getFilename())
 
     @staticmethod
     def canHaveExternalImports():
@@ -583,6 +610,10 @@ class UncompiledPythonModule(PythonModuleBase):
 
         self.used_modules = ()
 
+    def finalize(self):
+        del self.used_modules
+        del self.bytecode
+
     @staticmethod
     def isUncompiledPythonModule():
         return True
@@ -761,6 +792,9 @@ class PythonShlibModule(PythonModuleBase):
 
         self.used_modules = None
 
+    def finalize(self):
+        del self.used_modules
+
     def getDetails(self):
         return {
             "name"         : self.name,
@@ -838,51 +872,3 @@ class PythonShlibModule(PythonModuleBase):
 
     def getParentModule(self):
         return self
-
-
-class ExpressionModuleFileAttributeRef(ExpressionBase):
-    kind = "EXPRESSION_MODULE_FILE_ATTRIBUTE_REF"
-
-    def __init__(self, source_ref):
-        ExpressionBase.__init__(
-            self,
-            source_ref = source_ref
-        )
-
-    def mayRaiseException(self, exception_type):
-        return False
-
-    def computeExpressionRaw(self, trace_collection):
-        # There is not a whole lot to do here, the path will change at run
-        # time
-        if Options.getFileReferenceMode() != "runtime":
-            result = makeConstantRefNode(
-                constant   = self.getRunTimeFilename(),
-                source_ref = self.getSourceReference()
-            )
-
-            return result, "new_expression", "Using original '__file__' value."
-
-        return self, None, None
-
-    def getCompileTimeFilename(self):
-        return self.getParentModule().getCompileTimeFilename()
-
-    def getRunTimeFilename(self):
-        return self.getParentModule().getRunTimeFilename()
-
-
-class ExpressionModuleLoaderRef(ExpressionBase):
-    kind = "EXPRESSION_MODULE_LOADER_REF"
-
-    def __init__(self, source_ref):
-        ExpressionBase.__init__(
-            self,
-            source_ref = source_ref
-        )
-
-    def mayRaiseException(self, exception_type):
-        return False
-
-    def computeExpressionRaw(self, trace_collection):
-        return self, None, None
