@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -31,6 +31,7 @@ from logging import warning
 from nuitka.Builtins import builtin_names
 from nuitka.Constants import isConstant
 from nuitka.Options import isDebug, shallWarnImplicitRaises
+from nuitka.PythonVersions import python_version
 
 
 def makeConstantReplacementNode(constant, node):
@@ -41,6 +42,7 @@ def makeConstantReplacementNode(constant, node):
         source_ref = node.getSourceReference()
     )
 
+
 def makeRaiseExceptionReplacementExpression(expression, exception_type,
                                             exception_value):
     from .ExceptionNodes import ExpressionRaiseException
@@ -53,7 +55,7 @@ def makeRaiseExceptionReplacementExpression(expression, exception_type,
     if shallWarnImplicitRaises():
         warning(
             '%s: Will always raise exception: "%s(%s)"',
-            expression.getSourceReference().getAsString(),
+            source_ref.getAsString(),
             exception_type,
             exception_value
         )
@@ -73,6 +75,40 @@ def makeRaiseExceptionReplacementExpression(expression, exception_type,
     return result
 
 
+def makeRaiseExceptionReplacementStatement(statement, exception_type,
+                                           exception_value):
+    from .ExceptionNodes import StatementRaiseExceptionImplicit
+    from .BuiltinRefNodes import ExpressionBuiltinExceptionRef
+
+    source_ref = statement.getSourceReference()
+
+    assert type(exception_type) is str
+
+    if shallWarnImplicitRaises():
+        warning(
+            '%s: Will always raise exception: "%s(%s)"',
+            source_ref.getAsString(),
+            exception_type,
+            exception_value
+        )
+
+    result = StatementRaiseExceptionImplicit(
+        exception_type  = ExpressionBuiltinExceptionRef(
+            exception_name = exception_type,
+            source_ref     = source_ref
+        ),
+        exception_value = makeConstantReplacementNode(
+            constant = exception_value,
+            node     = statement
+        ),
+        exception_cause = None,
+        exception_trace = None,
+        source_ref      = source_ref
+    )
+
+    return result
+
+
 def makeRaiseExceptionReplacementExpressionFromInstance(expression, exception):
     assert isinstance(exception, Exception)
 
@@ -96,6 +132,13 @@ def makeRaiseExceptionExpressionFromTemplate(exception_type, template,
     from .BuiltinRefNodes import ExpressionBuiltinExceptionRef
     from .OperatorNodes import makeBinaryOperationNode
     from .ConstantRefNodes import makeConstantRefNode
+    from .ContainerMakingNodes import ExpressionMakeTuple
+
+    if type(template_args) is tuple:
+        template_args = ExpressionMakeTuple(
+            elements   = template_args,
+            source_ref = source_ref
+        )
 
     return ExpressionRaiseException(
         exception_type  = ExpressionBuiltinExceptionRef(
@@ -115,6 +158,7 @@ def makeRaiseExceptionExpressionFromTemplate(exception_type, template,
         source_ref      = source_ref
     )
 
+
 def makeRaiseTypeErrorExceptionReplacementFromTemplateAndValue(template,
                                                                operation,
                                                                original_node,
@@ -126,7 +170,7 @@ def makeRaiseTypeErrorExceptionReplacementFromTemplateAndValue(template,
     if type_name is not None:
         result = makeRaiseExceptionReplacementExpressionFromInstance(
             expression = original_node,
-            exception  = TypeError(template % type_name)
+            exception  = TypeError(template % type_name if '%' in template else template)
         )
 
         result = wrapExpressionWithNodeSideEffects(
@@ -172,9 +216,9 @@ def makeCompileTimeConstantReplacementNode(value, node):
         )
     elif type(value) is type:
         if value.__name__ in builtin_names:
-            from .BuiltinRefNodes import ExpressionBuiltinRef
+            from .BuiltinRefNodes import makeExpressionBuiltinRef
 
-            return ExpressionBuiltinRef(
+            return makeExpressionBuiltinRef(
                 builtin_name = value.__name__,
                 source_ref   = node.getSourceReference()
             )
@@ -183,13 +227,14 @@ def makeCompileTimeConstantReplacementNode(value, node):
     else:
         return node
 
+
 def getComputationResult(node, computation, description):
     """ With a computation function, execute it and return constant result or
         exception node.
 
     """
 
-    # Try and turn raised exceptions into static raises. pylint: disable=W0703
+    # Try and turn raised exceptions into static raises. pylint: disable=broad-except
     try:
         result = computation()
     except Exception as e:
@@ -264,18 +309,22 @@ def convertNoneConstantToNone(node):
     else:
         return node
 
+
 def wrapExpressionWithSideEffects(side_effects, old_node, new_node):
     assert new_node.isExpression()
 
     from .SideEffectNodes import ExpressionSideEffects
 
     if side_effects:
-        side_effects = [
-            side_effect
-            for side_effect in
-            side_effects
-            if side_effect.mayHaveSideEffects()
-        ]
+        side_effects = sum(
+            (
+                side_effect.extractSideEffects()
+                for side_effect in
+                side_effects
+                if side_effect.mayHaveSideEffects()
+            ),
+            ()
+        )
 
         if side_effects:
             new_node = ExpressionSideEffects(
@@ -404,25 +453,46 @@ def makeVariableRefNode(variable, source_ref):
         from .VariableRefNodes import ExpressionVariableRef
 
         return ExpressionVariableRef(
-            variable_name = variable.getName(),
-            variable      = variable,
-            source_ref    = source_ref
-        )
-
-
-def makeVariableTargetRefNode(variable, source_ref):
-    if variable.isTempVariable():
-        from .AssignNodes import ExpressionTargetTempVariableRef
-
-        return ExpressionTargetTempVariableRef(
             variable   = variable,
             source_ref = source_ref
         )
-    else:
-        from .AssignNodes import ExpressionTargetVariableRef
 
-        return ExpressionTargetVariableRef(
-            variable_name = variable.getName(),
-            variable      = variable,
-            source_ref    = source_ref
+
+def makeExpressionBuiltinLocals(provider, source_ref):
+    while provider.isExpressionOutlineFunction():
+        provider = provider.getParentVariableProvider()
+
+    if provider.isCompiledPythonModule():
+        from .GlobalsLocalsNodes import ExpressionBuiltinGlobals
+
+        return ExpressionBuiltinGlobals(
+            source_ref = source_ref
         )
+    else:
+        from .GlobalsLocalsNodes import (
+            ExpressionBuiltinLocalsCopy,
+            ExpressionBuiltinLocalsRef,
+            ExpressionBuiltinLocalsUpdated
+        )
+
+        if provider.isExpressionClassBody():
+            return ExpressionBuiltinLocalsRef(
+                locals_scope = provider.getFunctionLocalsScope(),
+                source_ref   = source_ref
+            )
+        elif python_version >= 300 or provider.isUnoptimized():
+            assert provider.getFunctionLocalsScope(), provider
+
+            return ExpressionBuiltinLocalsUpdated(
+                locals_scope = provider.getFunctionLocalsScope(),
+                source_ref   = source_ref
+            )
+        else:
+            # TODO: Make this not true, there ought to be always a locals
+            # scope.
+            assert not provider.getFunctionLocalsScope(), provider
+
+            return ExpressionBuiltinLocalsCopy(
+                locals_scope = provider.getFunctionLocalsScope(),
+                source_ref   = source_ref
+            )
