@@ -72,8 +72,9 @@ _python_version = None
 _python_arch = None
 _python_executable = None
 
-def setup(needs_io_encoding = False, silent = False):
-    goMainDir()
+def setup(needs_io_encoding = False, silent = False, go_main = True):
+    if go_main:
+        goMainDir()
 
     if "PYTHON" not in os.environ:
         os.environ["PYTHON"] = sys.executable
@@ -528,6 +529,14 @@ Error, needs 'strace' on your system to scan used libraries."""
         )
 
         _stdout_strace, stderr_strace = process.communicate()
+        exit_strace = process.returncode
+
+        if exit_strace != 0:
+            if str is not bytes:
+                stderr_strace = stderr_strace.decode("utf8")
+
+            my_print(stderr_strace, file = sys.stderr)
+            sys.exit("Failed to run strace.")
 
         open(path+".strace","wb").write(stderr_strace)
 
@@ -931,7 +940,7 @@ def checkDebugPython():
         my_print("Warning, using non-debug Python makes this test ineffective.")
         sys.gettotalrefcount = lambda : 0
     elif sys.version_info >= (3,7,0) and sys.version_info < (3,7,1):
-        my_print("Warning, bug of CPython 3.7.0 breaks reference counting and makes this test ineffective.")
+        my_print("Warning, bug of CPython 3.7.0/1 breaks reference counting and makes this test ineffective.")
         sys.gettotalrefcount = lambda : 0
 
 
@@ -1226,6 +1235,13 @@ def getTestingCacheDir():
     return result
 
 
+def getTestingCPythonOutputsCacheDir():
+    cache_dir = getCacheDir()
+
+    result = os.path.join(cache_dir, "cpython_outputs")
+    makePath(result)
+    return result
+
 @contextmanager
 def withDirectoryChange(path, allow_none = False):
     if path is not None or not allow_none:
