@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,8 +24,8 @@ from nuitka.nodes.AssignNodes import (
     StatementReleaseVariable
 )
 from nuitka.nodes.AttributeNodes import (
-    ExpressionAttributeLookup,
-    ExpressionBuiltinHasattr
+    ExpressionAttributeCheck,
+    ExpressionAttributeLookup
 )
 from nuitka.nodes.BuiltinDictNodes import ExpressionBuiltinDict
 from nuitka.nodes.BuiltinIteratorNodes import ExpressionBuiltinIter1
@@ -35,19 +35,14 @@ from nuitka.nodes.BuiltinRefNodes import (
     makeExpressionBuiltinRef
 )
 from nuitka.nodes.BuiltinTypeNodes import ExpressionBuiltinTuple
-from nuitka.nodes.CallNodes import (
-    ExpressionCall,
-    ExpressionCallEmpty,
-    ExpressionCallKeywordsOnly,
-    ExpressionCallNoKeywords
-)
+from nuitka.nodes.CallNodes import makeExpressionCall
 from nuitka.nodes.ComparisonNodes import (
     ExpressionComparisonIn,
     ExpressionComparisonIsNOT
 )
 from nuitka.nodes.ConditionalNodes import (
     ExpressionConditionalOR,
-    StatementConditional
+    makeStatementConditional
 )
 from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
 from nuitka.nodes.ContainerMakingNodes import ExpressionMakeTuple
@@ -57,14 +52,12 @@ from nuitka.nodes.ExceptionNodes import (
     StatementRaiseException
 )
 from nuitka.nodes.FunctionNodes import (
-    ExpressionFunctionBody,
     ExpressionFunctionCall,
     ExpressionFunctionCreation,
     ExpressionFunctionRef
 )
 from nuitka.nodes.LoopNodes import StatementLoop, StatementLoopBreak
 from nuitka.nodes.OperatorNodes import makeBinaryOperationNode
-from nuitka.nodes.ParameterSpecs import ParameterSpec
 from nuitka.nodes.ReturnNodes import StatementReturn
 from nuitka.nodes.SubscriptNodes import (
     ExpressionSubscriptLookup,
@@ -82,19 +75,20 @@ from nuitka.PythonVersions import (
     getComplexCallSequenceErrorTemplate,
     python_version
 )
+from nuitka.specs.ParameterSpecs import ParameterSpec
 
-from .Helpers import (
-    makeConditionalStatement,
-    makeStatementsSequenceFromStatement,
-    makeStatementsSequenceFromStatements
-)
 from .InternalModule import (
-    getInternalModule,
     internal_source_ref,
+    makeInternalHelperFunctionBody,
     once_decorator
 )
 from .ReformulationTryExceptStatements import makeTryExceptSingleHandlerNode
 from .ReformulationTryFinallyStatements import makeTryFinallyStatement
+from .TreeHelpers import (
+    makeCallNode,
+    makeStatementsSequenceFromStatement,
+    makeStatementsSequenceFromStatements
+)
 
 # TODO: Consider using ExpressionOutlineNodes for at least some of these
 # or their own helpers.
@@ -145,10 +139,8 @@ def getCallableNameDescBody():
     # else:
     #     return called_type.__name__ + " object"
 
-    result = ExpressionFunctionBody(
-        provider   = getInternalModule(),
+    result = makeInternalHelperFunctionBody(
         name       = helper_name,
-        doc        = None,
         parameters = ParameterSpec(
             ps_name          = helper_name,
             ps_normal_args   = (
@@ -158,9 +150,7 @@ def getCallableNameDescBody():
             ps_dict_star_arg = None,
             ps_default_count = 0,
             ps_kw_only_args  = ()
-        ),
-        flags      = set(),
-        source_ref = internal_source_ref
+        )
     )
 
     called_variable = result.getVariableForAssignment(
@@ -236,7 +226,7 @@ def getCallableNameDescBody():
             source_ref = internal_source_ref
         )
 
-        no_branch = makeConditionalStatement(
+        no_branch = makeStatementConditional(
             condition  = ExpressionBuiltinIsinstance(
                 instance   = ExpressionVariableRef(
                     variable   = called_variable,
@@ -272,7 +262,7 @@ def getCallableNameDescBody():
             source_ref = internal_source_ref
         )
 
-        no_branch = makeConditionalStatement(
+        no_branch = makeStatementConditional(
             condition  = ExpressionBuiltinIsinstance(
                 instance   = ExpressionVariableRef(
                     variable   = called_variable,
@@ -300,7 +290,7 @@ def getCallableNameDescBody():
 
     result.setBody(
         makeStatementsSequenceFromStatement(
-            statement = makeConditionalStatement(
+            statement = makeStatementConditional(
                 condition  = ExpressionBuiltinIsinstance(
                     instance   = ExpressionVariableRef(
                         variable   = called_variable,
@@ -349,7 +339,6 @@ def makeStarListArgumentErrorRaise(called_variable, star_list_variable):
                                         function_body = getCallableNameDescBody(),
                                         source_ref    = internal_source_ref
                                     ),
-                                    code_object  = None,
                                     defaults     = (),
                                     kw_defaults  = None,
                                     annotations  = None,
@@ -390,23 +379,23 @@ def makeStarListArgumentErrorRaise(called_variable, star_list_variable):
 def _makeStarListArgumentToTupleStatement(called_variable,
                                           star_list_variable):
     if python_version >= 350:
-        non_tuple_code = makeConditionalStatement(
+        non_tuple_code = makeStatementConditional(
             condition  = ExpressionConditionalOR(
-                left       = ExpressionBuiltinHasattr(
-                    object_arg = ExpressionVariableRef(
+                left       = ExpressionAttributeCheck(
+                    object_arg     = ExpressionVariableRef(
                         variable   = star_list_variable,
                         source_ref = internal_source_ref
                     ),
-                    name       = makeConstantRefNode("__iter__", internal_source_ref),
-                    source_ref = internal_source_ref
+                    attribute_name = "__iter__",
+                    source_ref     = internal_source_ref
                 ),
-                right      = ExpressionBuiltinHasattr(
-                    object_arg = ExpressionVariableRef(
+                right      = ExpressionAttributeCheck(
+                    object_arg     = ExpressionVariableRef(
                         variable   = star_list_variable,
                         source_ref = internal_source_ref
                     ),
-                    name       = makeConstantRefNode("__getitem__", internal_source_ref),
-                    source_ref = internal_source_ref
+                    attribute_name = "__getitem__",
+                    source_ref     = internal_source_ref
                 ),
                 source_ref = internal_source_ref
             ),
@@ -448,7 +437,7 @@ def _makeStarListArgumentToTupleStatement(called_variable,
             source_ref     = internal_source_ref
         )
 
-    return makeConditionalStatement(
+    return makeStatementConditional(
         condition  = ExpressionComparisonIsNOT(
             left       = ExpressionBuiltinType1(
                 value      = ExpressionVariableRef(
@@ -491,7 +480,6 @@ def _makeRaiseExceptionMustBeMapping(called_variable,
                                         function_body = getCallableNameDescBody(),
                                         source_ref    = internal_source_ref
                                     ),
-                                    code_object  = None,
                                     defaults     = (),
                                     kw_defaults  = None,
                                     annotations  = None,
@@ -596,15 +584,15 @@ def _makeStarDictArgumentToDictStatement(result, called_variable,
         makeTryExceptSingleHandlerNode(
             tried          = StatementAssignmentVariable(
                 variable   = tmp_keys_variable,
-                source     = ExpressionCallEmpty(
-                    called     = _makeNameAttributeLookup(
+                source     = makeCallNode(
+                    _makeNameAttributeLookup(
                         ExpressionVariableRef(
                             variable   = star_dict_variable,
                             source_ref = internal_source_ref
                         ),
                         attribute_name = "keys"
                     ),
-                    source_ref = internal_source_ref
+                    internal_source_ref
                 ),
                 source_ref = internal_source_ref
             ),
@@ -650,7 +638,7 @@ def _makeStarDictArgumentToDictStatement(result, called_variable,
         ),
     )
 
-    tried = StatementConditional(
+    tried = makeStatementConditional(
         condition  = ExpressionComparisonIsNOT(
             left       = ExpressionBuiltinType1(
                 value      = ExpressionVariableRef(
@@ -697,6 +685,51 @@ def _makeStarDictArgumentToDictStatement(result, called_variable,
     )
 
 
+def _makeRaiseNoStringItem(called_variable):
+    return StatementRaiseException(
+        exception_type  = ExpressionBuiltinMakeException(
+            exception_name = "TypeError",
+            args           = (
+                makeBinaryOperationNode(
+                    operator   = "Mod",
+                    left       =  makeConstantRefNode(
+                        constant      = """\
+%s keywords must be strings""",
+                        source_ref    = internal_source_ref,
+                        user_provided = True
+                    ),
+                    right      = ExpressionFunctionCall(
+                        function   = ExpressionFunctionCreation(
+                            function_ref = ExpressionFunctionRef(
+                                function_body = getCallableNameDescBody(
+                                ),
+                                source_ref    = internal_source_ref
+                            ),
+                            defaults     = (),
+                            kw_defaults  = None,
+                            annotations  = None,
+                            source_ref   = internal_source_ref
+                        ),
+                        values     = (
+                            ExpressionVariableRef(
+                                variable   = called_variable,
+                                source_ref = internal_source_ref
+                            ),
+                        ),
+                        source_ref = internal_source_ref
+                    ),
+                    source_ref = internal_source_ref
+                ),
+            ),
+            source_ref     = internal_source_ref
+        ),
+        exception_value = None,
+        exception_trace = None,
+        exception_cause = None,
+        source_ref      = internal_source_ref
+    )
+
+
 def _makeRaiseDuplicationItem(called_variable, tmp_key_variable):
     return StatementRaiseException(
         exception_type  = ExpressionBuiltinMakeException(
@@ -719,7 +752,6 @@ def _makeRaiseDuplicationItem(called_variable, tmp_key_variable):
                                         ),
                                         source_ref    = internal_source_ref
                                     ),
-                                    code_object  = None,
                                     defaults     = (),
                                     kw_defaults  = None,
                                     annotations  = None,
@@ -782,7 +814,7 @@ def _makeStarDictArgumentMergeToKwStatement(result, called_variable, kw_variable
     ]
 
     mapping_loop_body = (
-        StatementConditional(
+        makeStatementConditional(
             condition  = ExpressionComparisonIn(
                 left       = ExpressionTempVariableRef(
                     variable   = tmp_key_variable,
@@ -794,11 +826,9 @@ def _makeStarDictArgumentMergeToKwStatement(result, called_variable, kw_variable
                 ),
                 source_ref = internal_source_ref
             ),
-            yes_branch = makeStatementsSequenceFromStatement(
-                statement = _makeRaiseDuplicationItem(
-                    called_variable  = called_variable,
-                    tmp_key_variable = tmp_key_variable
-                )
+            yes_branch = _makeRaiseDuplicationItem(
+                called_variable  = called_variable,
+                tmp_key_variable = tmp_key_variable
             ),
             no_branch  = None,
             source_ref = internal_source_ref
@@ -831,15 +861,15 @@ def _makeStarDictArgumentMergeToKwStatement(result, called_variable, kw_variable
         makeTryExceptSingleHandlerNode(
             tried          = StatementAssignmentVariable(
                 variable   = tmp_keys_variable,
-                source     = ExpressionCallEmpty(
-                    called     = _makeNameAttributeLookup(
+                source     = makeCallNode(
+                    _makeNameAttributeLookup(
                         ExpressionVariableRef(
                             variable   = star_dict_variable,
                             source_ref = internal_source_ref
                         ),
                         attribute_name = "keys"
                     ),
-                    source_ref = internal_source_ref
+                    internal_source_ref
                 ),
                 source_ref = internal_source_ref
             ),
@@ -915,7 +945,7 @@ def _makeStarDictArgumentMergeToKwStatement(result, called_variable, kw_variable
             ),
             source_ref = internal_source_ref
         ),
-        StatementConditional(
+        makeStatementConditional(
             condition  = ExpressionComparisonIn(
                 left       = ExpressionTempVariableRef(
                     variable   = tmp_key_variable,
@@ -927,11 +957,9 @@ def _makeStarDictArgumentMergeToKwStatement(result, called_variable, kw_variable
                 ),
                 source_ref = internal_source_ref
             ),
-            yes_branch = makeStatementsSequenceFromStatement(
-                statement = _makeRaiseDuplicationItem(
-                    called_variable  = called_variable,
-                    tmp_key_variable = tmp_key_variable
-                )
+            yes_branch = _makeRaiseDuplicationItem(
+                called_variable  = called_variable,
+                tmp_key_variable = tmp_key_variable
             ),
             no_branch  = None,
             source_ref = internal_source_ref
@@ -977,8 +1005,8 @@ def _makeStarDictArgumentMergeToKwStatement(result, called_variable, kw_variable
         StatementAssignmentVariable(
             variable   = tmp_iter_variable,
             source     = ExpressionBuiltinIter1(
-                value      = ExpressionCallEmpty(
-                    called     = _makeNameAttributeLookup(
+                value      = makeCallNode(
+                    _makeNameAttributeLookup(
                         ExpressionVariableRef(
                             variable   = star_dict_variable,
                             source_ref = internal_source_ref
@@ -987,7 +1015,7 @@ def _makeStarDictArgumentMergeToKwStatement(result, called_variable, kw_variable
                                            if python_version < 300 else
                                          "items"
                     ),
-                    source_ref = internal_source_ref
+                    internal_source_ref
                 ),
                 source_ref = internal_source_ref
             ),
@@ -1000,19 +1028,17 @@ def _makeStarDictArgumentMergeToKwStatement(result, called_variable, kw_variable
         ),
     )
 
-    dict_case = makeStatementsSequenceFromStatement(
-        statement = StatementConditional(
-            condition  = ExpressionVariableRef(
-                variable   = star_dict_variable,
-                source_ref = internal_source_ref
-            ),
-            yes_branch = dict_case,
-            no_branch  = None,
+    dict_case = makeStatementConditional(
+        condition  = ExpressionVariableRef(
+            variable   = star_dict_variable,
             source_ref = internal_source_ref
-        )
+        ),
+        yes_branch = dict_case,
+        no_branch  = None,
+        source_ref = internal_source_ref
     )
 
-    tried = StatementConditional(
+    tried = makeStatementConditional(
         condition  = ExpressionComparisonIsNOT(
             left       = ExpressionBuiltinType1(
                 value      = ExpressionVariableRef(
@@ -1061,10 +1087,8 @@ def getFunctionCallHelperStarList():
 
     # Only need to check if the star argument value is a sequence and then
     # convert to tuple.
-    result = ExpressionFunctionBody(
-        provider   = getInternalModule(),
+    result = makeInternalHelperFunctionBody(
         name       = helper_name,
-        doc        = None,
         parameters = ParameterSpec(
             ps_name          = helper_name,
             ps_normal_args   = (
@@ -1074,9 +1098,7 @@ def getFunctionCallHelperStarList():
             ps_dict_star_arg = None,
             ps_default_count = 0,
             ps_kw_only_args  = ()
-        ),
-        flags      = set(),
-        source_ref = internal_source_ref
+        )
     )
 
     called_variable = result.getVariableForAssignment(
@@ -1092,7 +1114,7 @@ def getFunctionCallHelperStarList():
             star_list_variable = star_arg_list_variable,
         ),
         StatementReturn(
-            expression = ExpressionCallNoKeywords(
+            expression = makeExpressionCall(
                 called     = ExpressionVariableRef(
                     variable   = called_variable,
                     source_ref = internal_source_ref
@@ -1101,6 +1123,7 @@ def getFunctionCallHelperStarList():
                     variable   = star_arg_list_variable,
                     source_ref = internal_source_ref
                 ),
+                kw         = None,
                 source_ref = internal_source_ref
             ),
             source_ref = internal_source_ref
@@ -1153,10 +1176,8 @@ def getFunctionCallHelperKeywordsStarList():
 
     # Only need to check if the star argument value is a sequence and then
     # convert to tuple.
-    result = ExpressionFunctionBody(
-        provider   = getInternalModule(),
+    result = makeInternalHelperFunctionBody(
         name       = helper_name,
-        doc        = None,
         parameters = ParameterSpec(
             ps_name          = helper_name,
             ps_normal_args   = orderArgs(
@@ -1166,9 +1187,7 @@ def getFunctionCallHelperKeywordsStarList():
             ps_dict_star_arg = None,
             ps_default_count = 0,
             ps_kw_only_args  = ()
-        ),
-        flags      = set(),
-        source_ref = internal_source_ref
+        )
     )
 
     called_variable = result.getVariableForAssignment(
@@ -1189,7 +1208,7 @@ def getFunctionCallHelperKeywordsStarList():
             star_list_variable = star_arg_list_variable,
         ),
         StatementReturn(
-            expression = ExpressionCall(
+            expression = makeExpressionCall(
                 called     = ExpressionVariableRef(
                     variable   = called_variable,
                     source_ref = internal_source_ref
@@ -1258,10 +1277,8 @@ def getFunctionCallHelperPosStarList():
 
     # Only need to check if the star argument value is a sequence and then
     # convert to tuple.
-    result = ExpressionFunctionBody(
-        provider   = getInternalModule(),
+    result = makeInternalHelperFunctionBody(
         name       = helper_name,
-        doc        = None,
         parameters = ParameterSpec(
             ps_name          = helper_name,
             ps_normal_args   = (
@@ -1271,9 +1288,7 @@ def getFunctionCallHelperPosStarList():
             ps_dict_star_arg = None,
             ps_default_count = 0,
             ps_kw_only_args  = ()
-        ),
-        flags      = set(),
-        source_ref = internal_source_ref
+        )
     )
 
     called_variable = result.getVariableForAssignment(
@@ -1294,7 +1309,7 @@ def getFunctionCallHelperPosStarList():
             star_list_variable = star_arg_list_variable,
         ),
         StatementReturn(
-            expression = ExpressionCallNoKeywords(
+            expression = makeExpressionCall(
                 called     = ExpressionVariableRef(
                     variable   = called_variable,
                     source_ref = internal_source_ref
@@ -1311,6 +1326,7 @@ def getFunctionCallHelperPosStarList():
                     ),
                     source_ref = internal_source_ref
                 ),
+                kw         = None,
                 source_ref = internal_source_ref
             ),
             source_ref = internal_source_ref
@@ -1366,10 +1382,8 @@ def getFunctionCallHelperPosKeywordsStarList():
 
     # Only need to check if the star argument value is a sequence and then
     # convert to tuple.
-    result = ExpressionFunctionBody(
-        provider   = getInternalModule(),
+    result = makeInternalHelperFunctionBody(
         name       = helper_name,
-        doc        = None,
         parameters = ParameterSpec(
             ps_name          = helper_name,
             ps_normal_args   = orderArgs(
@@ -1379,9 +1393,7 @@ def getFunctionCallHelperPosKeywordsStarList():
             ps_dict_star_arg = None,
             ps_default_count = 0,
             ps_kw_only_args  = ()
-        ),
-        flags      = set(),
-        source_ref = internal_source_ref
+        )
     )
 
     called_variable = result.getVariableForAssignment(
@@ -1406,7 +1418,7 @@ def getFunctionCallHelperPosKeywordsStarList():
             star_list_variable = star_arg_list_variable,
         ),
         StatementReturn(
-            expression = ExpressionCall(
+            expression = makeExpressionCall(
                 called     = ExpressionVariableRef(
                     variable   = called_variable,
                     source_ref = internal_source_ref
@@ -1500,10 +1512,8 @@ def getFunctionCallHelperStarDict():
 
     # Only need to check if the star argument value is a sequence and then
     # convert to tuple.
-    result = ExpressionFunctionBody(
-        provider   = getInternalModule(),
+    result = makeInternalHelperFunctionBody(
         name       = helper_name,
-        doc        = None,
         parameters = ParameterSpec(
             ps_name          = helper_name,
             ps_normal_args   = (
@@ -1513,9 +1523,7 @@ def getFunctionCallHelperStarDict():
             ps_dict_star_arg = None,
             ps_default_count = 0,
             ps_kw_only_args  = ()
-        ),
-        flags      = set(),
-        source_ref = internal_source_ref
+        )
     )
 
     called_variable = result.getVariableForAssignment(
@@ -1533,11 +1541,12 @@ def getFunctionCallHelperStarDict():
             star_dict_variable = star_arg_dict_variable,
         ),
         StatementReturn(
-            expression = ExpressionCallKeywordsOnly(
+            expression = makeExpressionCall(
                 called     = ExpressionVariableRef(
                     variable   = called_variable,
                      source_ref = internal_source_ref
                 ),
+                args       = None,
                 kw         = ExpressionVariableRef(
                     variable   = star_arg_dict_variable,
                     source_ref = internal_source_ref
@@ -1608,10 +1617,8 @@ def getFunctionCallHelperPosStarDict():
 
     # Only need to check if the star argument value is a sequence and then
     # convert to tuple.
-    result = ExpressionFunctionBody(
-        provider   = getInternalModule(),
+    result = makeInternalHelperFunctionBody(
         name       = helper_name,
-        doc        = None,
         parameters = ParameterSpec(
             ps_name          = helper_name,
             ps_normal_args   = (
@@ -1621,9 +1628,7 @@ def getFunctionCallHelperPosStarDict():
             ps_dict_star_arg = None,
             ps_default_count = 0,
             ps_kw_only_args  = ()
-        ),
-        flags      = set(),
-        source_ref = internal_source_ref
+        )
     )
 
     called_variable = result.getVariableForAssignment(
@@ -1645,7 +1650,7 @@ def getFunctionCallHelperPosStarDict():
             star_dict_variable = star_arg_dict_variable,
         ),
         StatementReturn(
-            expression = ExpressionCall(
+            expression = makeExpressionCall(
                 called     = ExpressionVariableRef(
                     variable   = called_variable,
                     source_ref = internal_source_ref
@@ -1753,10 +1758,8 @@ def getFunctionCallHelperKeywordsStarDict():
 
     # Only need to check if the star argument value is a sequence and then
     # convert to tuple.
-    result = ExpressionFunctionBody(
-        provider   = getInternalModule(),
+    result = makeInternalHelperFunctionBody(
         name       = helper_name,
-        doc        = None,
         parameters = ParameterSpec(
             ps_name          = helper_name,
             ps_normal_args   = (
@@ -1766,9 +1769,7 @@ def getFunctionCallHelperKeywordsStarDict():
             ps_dict_star_arg = None,
             ps_default_count = 0,
             ps_kw_only_args  = ()
-        ),
-        flags      = set(),
-        source_ref = internal_source_ref
+        )
     )
 
     called_variable = result.getVariableForAssignment(
@@ -1791,11 +1792,12 @@ def getFunctionCallHelperKeywordsStarDict():
             star_dict_variable = star_arg_dict_variable,
         ),
         StatementReturn(
-            expression = ExpressionCallKeywordsOnly(
+            expression = makeExpressionCall(
                 called     = ExpressionVariableRef(
                     variable   = called_variable,
                     source_ref = internal_source_ref
                 ),
+                args       = None,
                 kw         = ExpressionVariableRef(
                     variable   = kw_variable,
                     source_ref = internal_source_ref
@@ -1895,10 +1897,8 @@ def getFunctionCallHelperPosKeywordsStarDict():
 
     # Only need to check if the star argument value is a sequence and then
     # convert to tuple.
-    result = ExpressionFunctionBody(
-        provider   = getInternalModule(),
+    result = makeInternalHelperFunctionBody(
         name       = helper_name,
-        doc        = None,
         parameters = ParameterSpec(
             ps_name          = helper_name,
             ps_normal_args   = (
@@ -1908,9 +1908,7 @@ def getFunctionCallHelperPosKeywordsStarDict():
             ps_dict_star_arg = None,
             ps_default_count = 0,
             ps_kw_only_args  = ()
-        ),
-        flags      = set(),
-        source_ref = internal_source_ref
+        )
     )
 
     called_variable = result.getVariableForAssignment(
@@ -1937,7 +1935,7 @@ def getFunctionCallHelperPosKeywordsStarDict():
             star_dict_variable = star_arg_dict_variable,
         ),
         StatementReturn(
-            expression = ExpressionCall(
+            expression = makeExpressionCall(
                 called     = ExpressionVariableRef(
                     variable   = called_variable,
                     source_ref = internal_source_ref
@@ -2028,10 +2026,8 @@ def getFunctionCallHelperStarListStarDict():
 
     # Only need to check if the star argument value is a sequence and then
     # convert to tuple.
-    result = ExpressionFunctionBody(
-        provider   = getInternalModule(),
+    result = makeInternalHelperFunctionBody(
         name       = helper_name,
-        doc        = None,
         parameters = ParameterSpec(
             ps_name          = helper_name,
             ps_normal_args   = (
@@ -2041,9 +2037,7 @@ def getFunctionCallHelperStarListStarDict():
             ps_dict_star_arg = None,
             ps_default_count = 0,
             ps_kw_only_args  = ()
-        ),
-        flags      = set(),
-        source_ref = internal_source_ref
+        )
     )
 
     called_variable = result.getVariableForAssignment(
@@ -2068,7 +2062,7 @@ def getFunctionCallHelperStarListStarDict():
 
     statements.append(
         StatementReturn(
-            expression = ExpressionCall(
+            expression = makeExpressionCall(
                 called     = ExpressionVariableRef(
                     variable   = called_variable,
                     source_ref = internal_source_ref
@@ -2122,10 +2116,8 @@ def getFunctionCallHelperPosStarListStarDict():
 
     # Only need to check if the star argument value is a sequence and then
     # convert to tuple.
-    result = ExpressionFunctionBody(
-        provider   = getInternalModule(),
+    result = makeInternalHelperFunctionBody(
         name       = helper_name,
-        doc        = None,
         parameters = ParameterSpec(
             ps_name          = helper_name,
             ps_normal_args   = (
@@ -2135,9 +2127,7 @@ def getFunctionCallHelperPosStarListStarDict():
             ps_dict_star_arg = None,
             ps_default_count = 0,
             ps_kw_only_args  = ()
-        ),
-        flags      = set(),
-        source_ref = internal_source_ref
+        )
     )
 
     called_variable = result.getVariableForAssignment(
@@ -2169,7 +2159,7 @@ def getFunctionCallHelperPosStarListStarDict():
 
     statements.append(
         StatementReturn(
-            expression = ExpressionCall(
+            expression = makeExpressionCall(
                 called     = ExpressionVariableRef(
                     variable   = called_variable,
                     source_ref = internal_source_ref
@@ -2235,10 +2225,8 @@ def getFunctionCallHelperKeywordsStarListStarDict():
 
     # Only need to check if the star argument value is a sequence and then
     # convert to tuple.
-    result = ExpressionFunctionBody(
-        provider   = getInternalModule(),
+    result = makeInternalHelperFunctionBody(
         name       = helper_name,
-        doc        = None,
         parameters = ParameterSpec(
             ps_name          = helper_name,
             ps_normal_args   = orderArgs(
@@ -2248,9 +2236,7 @@ def getFunctionCallHelperKeywordsStarListStarDict():
             ps_dict_star_arg = None,
             ps_default_count = 0,
             ps_kw_only_args  = ()
-        ),
-        flags      = set(),
-        source_ref = internal_source_ref
+        )
     )
 
     called_variable = result.getVariableForAssignment(
@@ -2279,7 +2265,7 @@ def getFunctionCallHelperKeywordsStarListStarDict():
 
     statements.append(
         StatementReturn(
-            expression = ExpressionCall(
+            expression = makeExpressionCall(
                 called     = ExpressionVariableRef(
                     variable   = called_variable,
                     source_ref = internal_source_ref
@@ -2337,10 +2323,8 @@ def getFunctionCallHelperPosKeywordsStarListStarDict():
 
     # Only need to check if the star argument value is a sequence and then
     # convert to tuple.
-    result = ExpressionFunctionBody(
-        provider   = getInternalModule(),
+    result = makeInternalHelperFunctionBody(
         name       = helper_name,
-        doc        = None,
         parameters = ParameterSpec(
             ps_name          = helper_name,
             ps_normal_args   = orderArgs(
@@ -2350,9 +2334,7 @@ def getFunctionCallHelperPosKeywordsStarListStarDict():
             ps_dict_star_arg = None,
             ps_default_count = 0,
             ps_kw_only_args  = ()
-        ),
-        flags      = set(),
-        source_ref = internal_source_ref
+        )
     )
 
     called_variable = result.getVariableForAssignment(
@@ -2388,7 +2370,7 @@ def getFunctionCallHelperPosKeywordsStarListStarDict():
 
     statements.append(
         StatementReturn(
-            expression = ExpressionCall(
+            expression = makeExpressionCall(
                 called     = ExpressionVariableRef(
                     variable   = called_variable,
                     source_ref = internal_source_ref
@@ -2456,10 +2438,8 @@ def getFunctionCallHelperPosKeywordsStarListStarDict():
 def getFunctionCallHelperDictionaryUnpacking():
     helper_name = "complex_call_helper_dict_unpacking_checks"
 
-    result = ExpressionFunctionBody(
-        provider   = getInternalModule(),
+    result = makeInternalHelperFunctionBody(
         name       = helper_name,
-        doc        = None,
         parameters = ParameterSpec(
             ps_name          = helper_name,
             ps_normal_args   = (
@@ -2469,9 +2449,7 @@ def getFunctionCallHelperDictionaryUnpacking():
             ps_dict_star_arg = None,
             ps_default_count = 0,
             ps_kw_only_args  = ()
-        ),
-        flags      = set(),
-        source_ref = internal_source_ref
+        )
     )
 
     args_variable = result.getVariableForAssignment(
@@ -2490,7 +2468,28 @@ def getFunctionCallHelperDictionaryUnpacking():
     tmp_key_variable = result.allocateTempVariable(temp_scope, "dict_key")
 
     update_body = (
-        StatementConditional(
+        makeStatementConditional(
+            condition  = ExpressionComparisonIsNOT(
+                left       = ExpressionBuiltinType1(
+                    value      = ExpressionTempVariableRef(
+                        variable   = tmp_key_variable,
+                        source_ref = internal_source_ref
+                    ),
+                    source_ref = internal_source_ref
+                ),
+                right      = makeExpressionBuiltinRef(
+                    builtin_name = "str",
+                    source_ref   = internal_source_ref
+                ),
+                source_ref = internal_source_ref
+            ),
+            yes_branch = _makeRaiseNoStringItem(
+                called_variable = called_variable,
+            ),
+            no_branch  = None,
+            source_ref = internal_source_ref
+        ),
+        makeStatementConditional(
             condition  = ExpressionComparisonIn(
                 left       = ExpressionTempVariableRef(
                     variable   = tmp_key_variable,
@@ -2502,11 +2501,9 @@ def getFunctionCallHelperDictionaryUnpacking():
                 ),
                 source_ref = internal_source_ref
             ),
-            yes_branch = makeStatementsSequenceFromStatement(
-                statement = _makeRaiseDuplicationItem(
-                    called_variable  = called_variable,
-                    tmp_key_variable = tmp_key_variable
-                )
+            yes_branch = _makeRaiseDuplicationItem(
+                called_variable  = called_variable,
+                tmp_key_variable = tmp_key_variable
             ),
             no_branch  = None,
             source_ref = internal_source_ref
@@ -2542,15 +2539,15 @@ def getFunctionCallHelperDictionaryUnpacking():
                 StatementAssignmentVariable(
                     variable   = tmp_iter2_variable,
                     source     = ExpressionBuiltinIter1(
-                        value      = ExpressionCallEmpty(
-                            called     = _makeNameAttributeLookup(
+                        value      = makeCallNode(
+                            _makeNameAttributeLookup(
                                 ExpressionTempVariableRef(
                                     variable   = tmp_item_variable,
                                     source_ref = internal_source_ref
                                 ),
                                 attribute_name = "keys"
                             ),
-                            source_ref = internal_source_ref
+                            internal_source_ref
                         ),
                         source_ref = internal_source_ref
                     ),
@@ -2563,41 +2560,9 @@ def getFunctionCallHelperDictionaryUnpacking():
                 )
             ),
             exception_name = "AttributeError",
-            handler_body   = StatementRaiseException(
-                exception_type  = ExpressionBuiltinMakeException(
-                    exception_name = "TypeError",
-                    args           = (
-                        makeBinaryOperationNode(
-                            operator   = "Mod",
-                            left       =  makeConstantRefNode(
-                                constant      = """\
-'%s' object is not a mapping""",
-                                source_ref    = internal_source_ref,
-                                user_provided = True
-                            ),
-                            right      = ExpressionMakeTuple(
-                                elements   = (
-                                    _makeNameAttributeLookup(
-                                        ExpressionBuiltinType1(
-                                            value      = ExpressionTempVariableRef(
-                                                variable   = tmp_item_variable,
-                                                source_ref = internal_source_ref
-                                            ),
-                                            source_ref = internal_source_ref
-                                        )
-                                    ),
-                                ),
-                                source_ref = internal_source_ref
-                            ),
-                            source_ref = internal_source_ref
-                        ),
-                    ),
-                    source_ref     = internal_source_ref
-                ),
-                exception_value = None,
-                exception_trace = None,
-                exception_cause = None,
-                source_ref      = internal_source_ref
+            handler_body   = _makeRaiseExceptionMustBeMapping(
+                called_variable    = called_variable,
+                star_dict_variable = tmp_item_variable
             ),
             source_ref     = internal_source_ref
         ),
