@@ -909,11 +909,15 @@ def simpleFunction110():
     open("me", buffering = True)
     __builtins__.open = orig_open
 
+####################################
+
 u = u'__name__'
 
 def simpleFunction111():
     return getattr(simpleFunction111, u)
 
+####################################
+
 def simpleFunction112():
     TESTFN = "tmp.txt"
     import codecs
@@ -938,6 +942,16 @@ def simpleFunction112():
 
 ####################################
 
+def simpleFunction113():
+    class A(object):
+        pass
+    a = A()
+    a.a = a
+
+    return a
+
+
+####################################
 
 # These need stderr to be wrapped.
 tests_stderr = (63,)
